require("Resources")

function new_collider()
	local grid_size = 1

	-- Collision testing helper
	local function test_collision(e1, e2)
		local mesh1 = e1.mesh
		if mesh1 ~= nil then
			-- E1 is a mesh
			local rad2 = e2.radius
			if rad2 ~= nil then
				-- E2 is a sphere
				return mesh_test_sphere(mesh1, e2.pos, rad2, e1.inv_mtx)
			else
				-- E2 is a mesh
				-- (not implemented)
			end
		else
			-- E1 is a sphere
			local rad1 = e1.radius
			
			local mesh2 = e2.mesh
			if mesh2 ~= nil then
				-- E2 is a mesh
				return mesh_test_sphere(mesh2, e1.pos, rad1, e2.inv_mtx)
			else
				-- E2 is a sphere
				local rad2 = e2.radius
				
				return (vec3_distance(e1.pos, e2.pos) < (rad1 + rad2))
			end
		end
		return false
	end
	
	return
		{
			field = {},
		
			reset = function(this)
				this.field = {}
				end,
			
			add_object = function(this, game_obj)
				if game_obj == nil then
					return
				end
				
				if not game_obj.col_enabled then
					return
				end
				
				-- Get the collision position
				local col_mtx = game_obj:get_col_matrix()
				local col_pos = mat4_get_trn(col_mtx)
				local pos_x = vec3_x(col_pos)
				local pos_y = vec3_z(col_pos)
				
				-- Start building the entry
				local entry = { obj = game_obj, pos = col_pos }
				
				local col_mesh = resources:get(game_obj.col_mesh)
				local add_radius
				if col_mesh ~= nil then
					add_radius = mesh_get_radius(col_mesh)
					
					-- Mesh collision
					entry.mesh = col_mesh
					entry.inv_mtx = mat4_inv(col_mtx)
				else
					-- Normal sphere collision
					add_radius = game_obj.col_radius
					entry.radius = add_radius
				end

				-- Get the index range
				local x1 = math.floor((pos_x - add_radius) / grid_size)
				local x2 = math.floor((pos_x + add_radius) / grid_size)
				local y1 = math.floor((pos_y - add_radius) / grid_size)
				local y2 = math.floor((pos_y + add_radius) / grid_size)
				entry.first_x = x1
				entry.first_y = y1
				
				local field = this.field
				local amt = 0
				for y=y1,y2 do
					-- Get the y-line
					if field[y] == nil then
						field[y] = {}
					end
					local line = field[y]
				
					-- Add the x-line
					for x=x1,x2 do
						if line[x] == nil then
							line[x] = { entry }
						else
							table.insert(line[x], entry)
						end
						amt = amt + 1
					end
				end
				--print(amt, "pos:", x1, x2, y1, y2, "rad:", add_radius)
				
				end,
				
			do_collisions = function(this)
				local col_amt = 0
				
				for y,line in pairs(this.field) do
					for x,entries in pairs(line) do
						-- Run the collisions
						local amt = #entries
						for i1=1,amt do
							local e1 = entries[i1]
							for i2=i1+1,amt do
								local e2 = entries[i2]
								
								-- Test a pair only once; at the starting coords of either one
								local test_x = math.max(e1.first_x, e2.first_x)
								local test_y = math.max(e1.first_y, e2.first_y)
								if x == test_x and y == test_y and test_collision(e1, e2) then
									local obj1 = e1.obj
									local obj2 = e2.obj
									obj1:on_collision(obj2)
									obj2:on_collision(obj1)
									
									col_amt = col_amt + 1
								end
							end
						end
					end
					
					--print("Collisions:", col_amt)
				end
				end,
		}
end
