require("Material")
require("Math_Util")

function new_game_background()
	local spawn_z = 1300
	local kill_z = -500
	
	local ret =
		{
			cubes = {},
			
			add_timer = 0,
			
			add_cube = function(this, spawn_z)
				local mrs = 1
				local new_cube =
				{
					pos = vec3(rand_float(-800, 800), rand_float(-230, -400), spawn_z),
					ang = vec3(rand_float(0, math.pi*2), rand_float(0, math.pi*2), rand_float(0, math.pi*2)),
					vel = vec3(0, 0, rand_float(-60, -100)),
					ang_vel = vec3(rand_float(-mrs, mrs), rand_float(-mrs, mrs), rand_float(-mrs, mrs)),
					
					scale = rand_float(60, 180),
					mesh = "cube_smooth",
					material = new_material("material_normal"),
				}
				new_cube.material:set_shader_value("color", vec4(0.05, 0.025, 0.2, 1))
				
				table.insert(this.cubes, new_cube)
				end,
			
			update = function(this, delta_t)
				-- Add a new cube?
				this.add_timer = this.add_timer - delta_t
				if this.add_timer < 0 then
					this.add_timer = 1
					this:add_cube(spawn_z)
					end
					
				-- Update the cubes
				local new_cubes = {}
				for k,cube in pairs(this.cubes) do
					cube.pos = cube.pos + cube.vel * delta_t
					cube.ang = cube.ang + cube.ang_vel * delta_t
					
					if vec3_z(cube.pos) > -500 then
						table.insert(new_cubes, cube)
					end
				end
				this.cubes = new_cubes
				end,
				
			render = function(this, renderer)
				for k,cube in pairs(this.cubes) do
					local ang = cube.ang
					local mtx =
						mat4_translation(cube.pos) * 
						mat4_rot_xyz(ang[1], ang[2], ang[3]) *
						mat4_scale(cube.scale)
					renderer:add_mesh(cube.mesh, mtx, cube.material)
				end
				end,
		}
	
	-- Add initial cubes
	for i=1,30 do
		ret:add_cube(rand_float(spawn_z, kill_z))
	end
	
	return ret
end
