require("Bullet")
require("Game_Object")

function new_player(game_area)
	local base = new_game_object(game_area)
	
	return
		game_object_extend(base,
		{
			-- Type of game object
			type = "player",
			
			-- Collision radius
			col_radius = 1,
			
			-- Render mesh (for default render behavior)
			render_mesh = "ship",
			-- Render shader args
			render_args = { color = vec4(0.7,1.5,1.9,1) },
			
			-- Tilting up-vector
			tilt_up = vec3(0,1,0),
		
			is_alive = true,
		
			shoot_pause = 0,
		
			-- Returns the current rendering matrix
			get_render_matrix = function(this)
				return mat4_target_at(this.pos, vec3_add(this.pos, vec3(0,0,1)), this.tilt_up)
				end,
		
			-- Updates the object
			update = function(this, delta_t)
				-- Alive?
				if not this.is_alive then
					return
				end
				
				-- Read the controls
				local mouse_x, mouse_y, mouse_b1, mouse_b2 = mouse_read()
				
				-- Difficulty changing
				local diff_add = 0
				if mouse_b1 then diff_add = -1 end
				if mouse_b2 then diff_add = 1 end
				game_area.difficulty = clamp(game_area.difficulty + diff_add * 0.5 * delta_t, 0, 1)
				
				-- Move the ship
				local dest_pos = vec3((mouse_x - 0.5) * 2 * -game_area.size_x, 0.0, (mouse_y - 0.5) * 2 * -game_area.size_z)
				local old_pos = this.pos
				this.pos = lerp(this.pos, dest_pos, 0.3)
				local delta_pos = this.pos - old_pos
				
				-- Update the tilting
				local tilt_dest = vec3(0,1,0) + delta_pos * 1
				this.tilt_up = vec3_unit(lerp(this.tilt_up, tilt_dest, 0.1))
				
				-- Shooting
				this.shoot_pause = this.shoot_pause + delta_t
				if this.shoot_pause > 0.15 then
					this.shoot_pause = 0
					
					resources:get("sound_shoot"):play()
					
					local bullet = new_bullet(game_area, this.pos, vec3(0, 0, 1) * 100)
					game_area:add_bullet(bullet)

					bullet = new_bullet(game_area, this.pos, vec3(0.5, 0, 1):unit() * 100)
					game_area:add_bullet(bullet)

					bullet = new_bullet(game_area, this.pos, vec3(-0.5, 0, 1):unit() * 100)
					game_area:add_bullet(bullet)
				end
				end,

			-- Called on collision
			on_collision = function(this, oth_obj)
				if oth_obj.type == "bullet" then
					return
				end
				
				-- Mark as dead
				this.is_alive = false
				
				-- Play the death sound
				resources:get("sound_player_death"):play()
				end,
		})
end
